# frozen_string_literal: true

require 'securerandom'
require 'fast_blank'
require 'jekyll-write-and-commit-changes'

Jekyll::Hooks.register :site, :post_read, priority: :low do |site|
  documents = site.documents

  site.data['layouts']&.each do |name, layout|
    site.data.dig('migration', name)&.each do |row|
      row['date'] = Jekyll::Utils.parse_date(row['date']) unless row['date'].blank?

      if row['id']
        document = documents.find do |doc|
          doc.data['id'] == row['id']
        end
      end

      document ||=
        begin
          base = "#{row['date'] || Date.today.to_s}-#{Jekyll::Utils.slugify(row['title'], mode: 'latin')}.markdown"
          path = File.join(site.source, '_posts', base)

          raise ArgumentError, "Row #{row['id']} duplicates file #{base}" if File.exist? path

          doc = Jekyll::Document.new(path, site: site, collection: site.collections['posts'])
          site.collections['posts'] << doc

          doc
        end

      row.each do |attribute, value|
        row[attribute] =
          case layout.dig(attribute, 'type')
            when 'string' then value
            when 'text' then value
            when 'tel' then value
            when 'color' then value # TODO: validar
            when 'date' then Jekyll::Utils.parse_date(value)
            when 'email' then value # TODO: validar
            when 'url' then value # TODO: validar
            when 'content' then value
            when 'markdown_content' then value
            when 'markdown' then value
            when 'number' then value.to_i
            when 'order' then value.to_i
            when 'boolean' then !value.strip.empty?
            when 'array' then value.split(',').map(&:strip)
            # TODO: procesar los valores en base a los valores predefinidos
            when 'predefined_array' then value.split(',').map(&:strip)
            when 'image' then { 'path' => value, 'description' => '' }
            when 'file' then { 'path' => value, 'description' => '' }
            when 'geo' then %w[lat lng].zip(value.split(',', 2).map(&:to_f)).to_h
            when 'belongs_to' then value
            when 'has_many' then value.split(',').map(&:strip)
            when 'has_and_belongs_to_many' then value.split(',').map(&:strip)
            when 'related_posts' then value.split(',').map(&:strip)
            when 'locales' then value.split(',').map(&:strip)
            else value
          end
      end

      document.data['uuid'] ||= SecureRandom.uuid
      document.content = row.delete('content')

      document.data.merge! row
      document.save
    end
  end
end
